import { DOMWidgetModel, DOMWidgetView } from '@jupyter-widgets/base';
export declare const OUTPUT_WIDGET_VERSION = "1.0.0";
export declare class OutputModel extends DOMWidgetModel {
    defaults(): any;
}
export declare class OutputView extends DOMWidgetView {
}
