import React from "react";

// redux
import ColumnSummary from "./ColumnSummary";

const SummaryView = (props: any) => {          
    return (
        <ColumnSummary />
    );
}

export default SummaryView;


